'==========================================================================
'CRV2XYZ.BAS
'
'This script transforms cylindrical or spherical coordinates to XYZ

'	                        Original by Mark Sloan, GSI, July 23, 1996                           
'		                    Additional modifications TB - 11 JUL 96     
'
'
'												SKP 11/99 Surfer 7
'												SKP 4/2009 Surfer 9
'==========================================================================

Sub Main
	Dim SurferApp As Object
	Set SurferApp = CreateObject("Surfer.Application")
	SurferApp.Visible = True
	
	'Input file locations
	GetFileInfo(FileIn$, FileOut$)

	'Open FileIn$ and FileOut$ as Worksheet Documents
	Dim Wks As Object
	Set Wks = SurferApp.Documents.Open(FileIn$)
	Dim Wks2 As Object
	Set Wks2 = SurferApp.Documents.Add(srfDocWks)

	'Determine whether using cylindrical or spherical coordinates
	GetCoordinateSystem(SystemIn$) 

	'Shows drawing of coordinate system if user chooses to see drawings
	SeeDrawing% = MsgBox("Would you like to see a drawing of the coordinate system for this program?", _
		36,"See Coordinate System Drawing?")
	Dim Plot As Object
	If SeeDrawing% = 6 Then 
		GetDrawingDirectory (Directory$)
		'Systemin$ input (0=cylindrical, 1=spherical)
		If SystemIn$ = "0" Then 
			Set Plot = SurferApp.Documents.Open(Directory$+"\cylinder.srf")
			Plot.Windows(1).Zoom(srfZoomFitToWindow)		
		Else 
			Set Plot = SurferApp.Documents.Open(Directory$+"\sphere.srf")
			Plot.Windows(1).Zoom(srfZoomFitToWindow)		
		End If
		Void% = MsgBox("Please <Alt> + <Tab> to Surfer and examine picture" + _
			Chr$(13) + "When you are Finished, Return to this box and" + Chr$(13) + _
			"Click OK to continue.",48,"Examine Drawing in Surfer")
	End If

	'Get Worksheet Columns
	If SystemIn$ = "0" Then
		'Gets columns for cylindrical
		Prompt$ = "Enter the columns containing R, Theta, and Z where R is the distance from the Z axis to the point, Theta is the angle between the Z axis and the line connecting the origin and the point, and Z is Z"
		GetWksCol (Prompt$,RCol,ThetaCol,ZorPhiCol)	
	Else  
		'Gets columns for spherical
		Prompt$ = "Enter the columns containing R, Theta, and Z where R is the distance from the Z axis to the point, Theta is the angle between the Z axis and the line connecting the origin and the point, and Phi is the angle between the X axis and the line connecting the origin and the point"
		GetWksCol (Prompt$,RCol,ThetaCol,ZorPhiCol)	
	End If
	
	'Determine if Angle is in Radians or Degrees
	GetAngleUnits (AngleCheck%)
	If AngleCheck% = 0 Then AngCorrect# = 1
	If AngleCheck% = 1 Then AngCorrect# = 0.0174533
	
	'Copies cylindrical or spherical coordinates into XYZ file
	'Copies R value
	Wks.Columns(RCol).Copy
	Wks2.Columns("A").Paste
	'Copies Theta value
	Wks.Columns(ThetaCol).Copy
	Wks2.Columns("B").Paste
	'Copies Z or Phi value	
	Wks.Columns(ZorPhiCol).Copy
	Wks2.Columns("C").Paste
		
'	This section removed as Wks.Columns(1).RowCount will work to get the last row 
'	Changes LastRow% to lastrow read
'	Row% = 1  'Initialize Row% to 0 (Start at beginning of file)
'	While Wks.Cells(Row%,RCol)<>"" And Wks.Cells(Row%,ThetaCol)<>"" And Wks.Cells(Row%,ZorPhiCol)<>""
'		'Increments row number to move to next row
'		Row% = Row% + 1
'	Wend
'	LastRow% = Row%
		
	'Create X, Y, Z values by transforming data
	If SystemIn$ = "0" Then
		'This works for cylindrical coordinates
		If AngCorrect#=1 Then
			'These calculations are used when the angles are in radians
			Wks2.Transform(1, Wks.Columns(1).RowCount, "D=A*Cos(B)")
			Wks2.Transform(1, Wks.Columns(1).RowCount, "E=A*Sin(B)")
			Wks2.Transform(1, Wks.Columns(1).RowCount, "F=C")
		Else
			'These calculations are used when the angles are in degrees
			Wks2.Transform(1, Wks.Columns(1).RowCount, "D=A*Cos(0.0174533*B)")
			Wks2.Transform(1, Wks.Columns(1).RowCount, "E=A*Sin(0.0174533*B)")
			Wks2.Transform(1, Wks.Columns(1).RowCount, "F=C")
		End If
	Else
		'This works for spherical coordinates
		If AngCorrect#=1 Then
			'These calculations are used when the angles are in radians
			Wks2.Transform(1, Wks.Columns(1).RowCount, "D=A*Sin(B)*Cos(C)")
			Wks2.Transform(1, Wks.Columns(1).RowCount, "E=A*Sin(B)*Sin(C)")
			Wks2.Transform(1, Wks.Columns(1).RowCount, "F=A*Cos(B)")
		Else
			'These calculations are used when the angles are in radians
			Wks2.Transform(1, Wks.Columns(1).RowCount, "D=A*Sin(0.0174533*B)*Cos(0.0174533*C)")
			Wks2.Transform(1, Wks.Columns(1).RowCount, "E=A*Sin(0.0174533*B)*Sin(0.0174533*C)")
			Wks2.Transform(1, Wks.Columns(1).RowCount, "F=A*Cos(0.0174533*B)")
		End If
	End If	
	
	'Remove copied R, Theta, and ZorPhi Columns
	Wks2.Columns(1,3).Delete(wksDeleteLeft)
	
	'Place Header in top row if cells are empty
	If Wks2.Cells(1,1).Value = "" And Wks2.Cells(1,2).Value = "" And Wks2.Cells(1,3).Value = "" Then 
		Wks2.Cells(1,1).Value = "X"
		Wks2.Cells(1,2).Value = "Y"
		Wks2.Cells(1,3).Value = "Z"
	Else
		'Place Header in inserted top row if cells contain data 
		Wks2.Rows(1).Insert(wksInsertDown)
		Wks2.Cells(1,1).Value = "X"
		Wks2.Cells(1,2).Value = "Y"
		Wks2.Cells(1,3).Value = "Z"
	End If
		
	Debug.Print "The conversion was successful.  Last row processed was "; Wks.Columns(1).RowCount          
	Wks.Close(srfSaveChangesNo)				  'Close Input file
	'If output filename not defined previously, enter output filename
	While FileOut$ = "" 
		FileOut$=InputBox$("Enter path and filename of output file", "Output File Name?")
	Wend
	Wks2.Close(srfSaveChangesYes,FileOut$)	'Close Output file and Save changes
	Close
	SurferApp.Quit
End Sub
'Functions follow for dialog boxes
'
Private Function GetFileInfo(FileIn$, FileOut$) As String
	Begin Dialog UserDialog 400,119,"Enter File Locations" ' %GRID:10,7,1,1
		Text 10,20,90,14,"Input Data File",.Text1
		Text 10,50,90,14,"Export Data File",.Text2
		TextBox 100,20,270,21,.TextBox1
		TextBox 100,50,270,21,.TextBox2
		OKButton 100,90,90,20
		CancelButton 220,90,90,20,.cancel
	End Dialog
	Dim dlgvar As UserDialog
		
	'Display the dialog and wait for the OK or Cancel button to be pressed
	While Dir$(FileIn$,vbNormal) = "" 
		'Dialog dlgvar
		'FileIn$=dlgvar.TextBox1
		'FileOut$=dlgvar.TextBox2
		FileIn$=GetFilePath("cylindrical.dat","dat",,"Input Data File")
		FileOut$=GetFilePath("xyz.dat","dat",,"Output XYZ Data File",3)
		'Extract the information entered into the dialog
		If FileOut$ = "" Then MsgBox("Output Filename not input correctly", 0, "Missing File")
		If FileIn$ = "" Then MsgBox("Input Filename not input correctly", 0, "Missing File")
	Wend
End Function
'
Private Function GetCoordinateSystem (SystemIn$) As String
	Begin Dialog UserDialog 320,98,"Convert From" ' %GRID:10,7,1,1
	Text 20,14,270,21,"What system are you converting FROM?"
	OptionGroup .options
		OptionButton 40,40,90,15,"Cylindrical"
		OptionButton 160,40,90,15,"Spherical"
		OKButton 110,63,90,21
	End Dialog
	Dim dlg As UserDialog
	dlg.options = 1
    Dialog dlg ' show dialog (wait for ok)
    If dlg.options = 1 Then SystemIn$ = "1"
    If dlg.options = 0 Then SystemIn$ = "0"
End Function
'
Private Function GetDrawingDirectory (Directory$) As String
'Define the dialog template.  This definition is inserted by the UserDialog editor.
	Begin Dialog UserDialog 400,133,"Where are the SRF files?" ' %GRID:10,7,1,1
		OKButton 80,98,90,21
		CancelButton 200,98,90,21
		Text 30,14,320,35,"In what directory Are the SRF files (ie cylinder.srf, sphere.srf) accompanying this script located?",.Text1
		TextBox 30,56,320,21,.Text2
	End Dialog
	
	'Declare a dialog variable
	Dim dlgvar As UserDialog
	
	'Display the dialog and wait for the OK or Cancel button to be pressed
	result = Dialog(dlgvar)   
	
	'Extract the information entered into the dialog
	Directory$=dlgvar.Text2
End Function
'GetWksCol (Prompt$,RCol,ThetaCol,ZorPhiCol)
Private Function GetWksCol (Prompt$,RCol,ThetaCol,ZorPhiCol) As String
'Define the dialog template.  This definition is inserted by the UserDialog editor.
	Begin Dialog UserDialog 400,238,"Enter Column Information" ' %GRID:10,7,1,1
		Text 30,7,320,98,Prompt$,.Text1
		Text 20,110,90,14,"R Column",.Text2
		TextBox 110,110,240,21,.TextBox1
		Text 20,140,90,14,"Theta Column",.Text3
		TextBox 110,140,240,21,.TextBox2
		Text 20,170,90,14,"Z or R Column",.Text4
		TextBox 110,170,240,21,.TextBox3
		OKButton 90,210,90,21
		CancelButton 210,210,90,21
	End Dialog
	
	'Declare a dialog variable
	Dim dlgvar As UserDialog
	
	'Display the dialog and wait for the OK or Cancel button to be pressed
	result = Dialog(dlgvar)   
	
	'Extract the information entered into the dialog
	RowCol$=dlgvar.TextBox1
	ThCol$=dlgvar.TextBox2
	ZCol$=dlgvar.TextBox3
	
	'An empty entry or pressing Cancel ends the script
	If RowCol$ = "" Then Debug.Print "Program Terminated":End

	'Convert from column letter to column # (ie A=1) via ANSI code (lcase$ makes this conversion case insensitive)
	GetRCol = Asc(LCase$(RowCol$))
	RCol = GetRCol-96
	GetThetaCol = Asc(LCase$(ThCol$))
	ThetaCol = GetThetaCol-96
	GetZCol = Asc(LCase$(ZCol$))
	ZorPhiCol = GetZCol-96
End Function
'
Private Function GetAngleUnits (AngleCheck%) As String
	Begin Dialog UserDialog 290,98,"Radians or Degrees?" ' %GRID:10,7,1,1
		Text 40,14,250,21,"Is the data in Radians or Degrees?"
		OptionGroup .options
			OptionButton 40,40,90,15,"Radians"
			OptionButton 160,40,90,15,"Degrees"
		OKButton 110,63,90,21
	End Dialog
	Dim dlg As UserDialog
	dlg.options = 1
    Dialog dlg ' show dialog (wait for ok)
    If dlg.options = 1 Then AngleCheck% = 1
    If dlg.options = 0 Then AngleCheck% = 0
End Function
